function create_corr_plots(var1,var1_name,var2,var2_name,Th,i,j,singlePlot)

offset = 0.2;

if size(var2,2) == 1
    doublePlot = 0;
    rows = 1;
    vals = var2;
    plot_idx = [1 2 3];
    ME = cell(2,1);
else
    rows  = 2;
    doublePlot = 1;
    vals = max(var2,[],2);
    plot_idx = [1 3 5 2 4 6];
    ME{1} = 'Max:';
    ME{2} = 'Mean:';
end

if nargin ~= 8
    singlePlot = 0;
end

% compute correlation and plot:
%figure
[co,p] = corr(var1,vals);
[co2,p2] = corr(var1,vals,'type','Spearman');

if ~singlePlot
    subplot(rows,3,plot_idx(1))
    plot(var1,vals,'bo');
    xlabel(var1_name)
    ylabel(var2_name)
end

if nargin < 6
    disp([num2str(co) ' (p = ' num2str(p) ') Correlation between ' char(var1_name) ' and ' char(var2_name)])
else
    disp([num2str(co) ' (p = ' num2str(p) ') Correlation between ' char(var1_name) ' and ' char(var2_name) ' (' num2str(i) ' ' num2str(j) ')'])
end


% compute and plot linear fit:
LM = fitlm(var1,vals);
Q = table2array(LM.Coefficients);
LI = [min(var1) max(var1)];
b = Q(1,1);
k = Q(2,1);
y = k*(linspace(LI(1),LI(2),10))+b;

if ~singlePlot
    hold on;plot(linspace(LI(1),LI(2),10),y,'k-')
    grid on
    title([{[ME{1} ' Pearson = ' num2str(co) ' p = ' num2str(round(p*100)/100)]};{['Spearman = ' num2str(co2) ' p = ' num2str(round(p2*100)/100)]}])
end

% residual plot:
resi1 = table2array(LM.Residuals);
fi1 = find(abs(resi1(:,4)) > Th);

if ~singlePlot
    subplot(rows,3,plot_idx(2));plot(resi1(:,4),'bo');ylabel('standardized residual')
    grid on;hold on;
    plot(1:length(var1),Th*ones(1,length(var1)),'k')
    plot(1:length(var1),-Th*ones(1,length(var1)),'k')
    yli = get(gca,'YLim');
    ylim([min(yli(1),-Th-offset) max(yli(2),Th+offset)])
    xlim([1 length(var1)])
    plot(fi1,resi1(fi1,4),'r*')
    subplot(rows,3,plot_idx(1));hold on;plot(var1(fi1),vals(fi1),'r*')
end

% remove outliers:
var1_no = var1;
vals_no = vals;
var1_no(fi1) = [];
vals_no(fi1) = [];

[co,p] = corr(var1_no,vals_no);
[co2,p2] = corr(var1_no,vals_no,'type','Spearman');

if ~singlePlot
    subplot(rows,3,plot_idx(3));
else
    plot(var1_no,vals_no,'ko','MarkerFaceColor','b','MarkerSize',8);
    xlabel(var1_name)
    ylabel(var2_name)
end

% compute and plot linear fit:
LM = fitlm(var1_no,vals_no);
Q = table2array(LM.Coefficients);
LI = [min(var1_no) max(var1_no)];
LI = [LI(1)-LI(1)*0.01 LI(2)+LI(2)*0.01];
b = Q(1,1);
k = Q(2,1);
y = k*(linspace(LI(1),LI(2),10))+b;
hold on;plot(linspace(LI(1),LI(2),10),y,'r-','LineWidth',2)
xlim(LI)
grid on
title([{[ME{1} ' Pearson = ' num2str(co) ' p = ' num2str(round(p*10000)/10000)]};{['Spearman = ' num2str(co2) ' p = ' num2str(round(p2*10000)/10000)]}])

disp([num2str(co) ' (p = ' num2str(p) ') Correlation between ' char(var1_name) ' and ' char(var2_name) ' after removing ' num2str(length(fi1)) ' outliers'])

if doublePlot
    
    vals = mean(var2,2);
    subplot(rows,3,plot_idx(4))
    plot(var1,vals,'bo');
    xlabel(var1_name)
    ylabel(var2_name)
    [co,p] = corr(var1,vals);
    [co2,p2] = corr(var1,vals,'type','Spearman');
    LM = fitlm(var1,vals);
    Q = table2array(LM.Coefficients);
    LI = [min(var1) max(var1)];
    b = Q(1,1);
    k = Q(2,1);
    y = k*(linspace(LI(1),LI(2),10))+b;
    hold on;plot(linspace(LI(1),LI(2),10),y,'k-')
    grid on
    grid on;
    
    title([{[ME{2} ' Pearson = ' num2str(co) ' p = ' num2str(round(p*100)/100)]};{['Spearman = ' num2str(co2) ' p = ' num2str(round(p2*100)/100)]}])
    
    resi2 = table2array(LM.Residuals);
    subplot(rows,3,plot_idx(5));plot(resi2(:,4),'bo');ylabel('standardized residual');
    grid on;hold on;
    plot(1:length(var1),Th*ones(1,length(var1)),'k')
    plot(1:length(var1),-Th*ones(1,length(var1)),'k')
    yli = get(gca,'YLim');
    ylim([min(yli(1),-Th-offset) max(yli(2),Th+offset)])
    fi2 = find(abs(resi2(:,4)) > Th);
    xlim([1 length(var1)])
    plot(fi2,resi2(fi2,4),'r*')
    
    subplot(rows,3,plot_idx(4));hold on;plot(var1(fi2),vals(fi2),'r*')
    
    % remove outliers:
    var1_no = var1;
    vals_no = vals;
    var1_no(fi2) = [];
    vals_no(fi2) = [];
    
    subplot(rows,3,plot_idx(6));
    plot(var1_no,vals_no,'bo');
    [co,p] = corr(var1_no,vals_no);
    [co2,p2] = corr(var1_no,vals_no,'type','Spearman');
    xlabel(var1_name)
    ylabel(var2_name)
    
    % compute and plot linear fit:
    LM = fitlm(var1_no,vals_no);
    Q = table2array(LM.Coefficients);
    LI = [min(var1_no) max(var1_no)];
    b = Q(1,1);
    k = Q(2,1);
    y = k*(linspace(LI(1),LI(2),10))+b;
    hold on;plot(linspace(LI(1),LI(2),10),y,'k-')
    grid on
    title([{[ME{2} ' Pearson = ' num2str(co) ' p = ' num2str(round(p*100)/100)]};{['Spearman = ' num2str(co2) ' p = ' num2str(round(p2*100)/100)]}])
    
    
end