function [epochs,labels,data_f,baselines] = create_epochs(data,triggers,winlen,nofiltering,Fs,locutoff,hicutoff,orde,delay_vals)

verbose = false;
figOn = 0;

% window length:
L = round(Fs*winlen);

nrClasses = size(triggers,1);

N = zeros(1,nrClasses);
for c = 1:nrClasses
    trig{c} = find(diff(triggers(c,:))>0);
    trig{c}(trig{c}+L-1 > size(data,2)) = [];
    N(c) = length(trig{c});
end
% delay correction:
if ~isempty(delay_vals)
    for c = 1:nrClasses
        dval = delay_vals(c);
        trig{c} = trig{c} + round(Fs*dval);
    end
end
% balance class labels:
for c = 1:nrClasses
    trig{c} = trig{c}(1:min(N));
end

if nofiltering
    data_f = data;
else
    [data_f,filtwts] = eegfilt(data,Fs,locutoff,hicutoff,0,orde);
    % [data_f,filtwts] = eegfilt(data,Fs,locutoff,hicutoff);
end

% [b,a] = butter(orde,locutoff/(Fs/2),'high');
% data_f = filtfilt(b,a,data);
% [b,a] = butter(orde,hicutoff/(Fs/2));
% data_f = filtfilt(b,a,data_f);

% C = round(203*rand) + 1;
if figOn
    C = 20;
    x = data_f(C,:);
    x2 = data(C,:);
    [pxx,f] = pwelch(x,L,L/2,512,Fs);
    [pxx2,f2] = pwelch(x2,L,L/2,512,Fs);
    
    figure
    subplot(211)
    plot(data(C,10000:20000),'r--');hold on
    plot(data_f(C,10000:20000),'b-');hold on
    subplot(212)
    hold on
    plot(f,10*log10(pxx2),'r--')
    plot(f,10*log10(pxx),'b-')
    legend([{'original','filtered'}])
    xlabel('Frequency (Hz)')
    ylabel('Magnitude (dB)')
    xlim([0 100])
end
labels = [];
for c = 1:nrClasses
    for m = 1:length(trig{c})
        if verbose
            disp(['Class ' num2str(c) ', epoch ' num2str(m) '/' num2str(length(trig{c}))])
        end
        datawindow = trig{c}(m):(trig{c}(m)+L-1);
        baseline_m = data_f(:,(datawindow(1)-round(Fs/2)):(datawindow(1)-1)); % save 0.5 second baseline
        epoch_m = data_f(:,datawindow);
        if m == 1 && c == 1
            epochs = epoch_m;
            baselines = baseline_m;
        else
            epochs = cat(3,epochs,epoch_m);
            baselines = cat(3,baselines,baseline_m);
        end
        labels = [labels c];
    end
end

epochs = permute(epochs,[3 2 1]);
baselines = permute(baselines,[3 2 1]);
