% This is the main script to perform decoding analysis described in the manuscript:
% Decoding proprioceptive and tactile stimuli from MEG signals: 
% a feasibility study and comparison of methods.
%
% jukka-pekka.kauppi@jyu.fi

% first go to the code directory and set all subdirectories to the path:
cd('C:\Users\jpkau\Seafile\Personal Documents\tutkimus\2018\MEG\koodit\codes')
addpath(genpath(cd))

% Provide source file name(s) in a cell-struct.
% length(sourceFilenames) must correspond to the total number of subjects in the analysis:
sourceFilenames{1} = ['C:\Users\jpkau\Seafile\Personal Documents\tutkimus\2018\MEG\TD26_mov_otp_tsss_mc_ica.fif'];

% Provide names of the decoding categories. 
% length(classNames) must correspond to the total number of classes:
classNames = [{'R.IndexFinger'},{'L.IndexFinger'},{'R.Ankle'},{'L.Ankle'}];

% Provide names of the trigger channels.
% % The number of channels must correspond to the number of classes:
triggerChannels = [{'STI001'},{'STI002'},{'STI003'},{'STI004'}];

% Give directory in which results will be saved:
result_dir = 'C:\Users\jpkau\Seafile\Personal Documents\tutkimus\2018\MEG\MEG_results\';

% Obtain default analysis parameters:
Params = init_decoding_params(sourceFilenames,classNames,result_dir,triggerChannels);

% Example how to change decoding parameters:
% Params.decodingParams.ica_on = 1; % run ICA
% Params.decodingParams.nrICs = 30; % choose number of ICs

% run decoding for subject 1:
subject_index = 1;

% Create or load epochs and class labels from 306 channel MEG time series
% data saved in fif format. If data is in other format, variables 
% "epochs" and "labels" need to be generated manually.
[epochs,labels] = prepareDatasetsMEG(subject_index,Params);

% perform decoding:
results = run_decoding(epochs,labels,subject_index,Params);

