function [DataCV,data,labels,outerLoopSize,subj_index] = divide_data_into_CV_blocks(data,labels,style,nFolds,LOSO,subj_index,saveMem)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Divide data into CV blocks:
switch style
    case 'normal'
        rp = randperm(size(data,1));
        data = data(rp,:,:);
        labels = labels(rp);
        if LOSO % leave-one-subject-out analysis:
            subj_index = subj_index(rp);
            un = unique(subj_index);
            % Use subject-wise blocks as different CV folds:
            BL = [];
            for bl = 1:length(un)
                BL{bl} = find(subj_index == un(bl));
            end
            DataCV = divide_data_into_blocks(data,labels,nFolds,'linear',0,BL);
        else
            DataCV = divide_data_into_blocks(data,labels,nFolds);
        end
        outerLoopSize = size(DataCV.data,1);
    case {'threewayDA','temporalthreewayDA','temporalthreewayDAfull',...
            'temporalthreewayRDA','temporalthreewaySDA',...
            'bilinear','temporalbilinear','temporalthreewayDAfullorig'}
        rp = randperm(size(data{1},3));
        labels = labels(rp);
        for m = 1:length(data)
            if strcmp(style,'threewayDA')
                data{m} = abs(data{m}(:,:,rp));
            elseif strcmp(style,'temporalbilinear')
                data{m} = data{m}(:,:,rp);
                for s = 1:size(data{m},3)
                    D = data{m}(:,:,s);
                    D = D - repmat(mean(D,2),1,size(D,2));
                    D = D./repmat(std(D,[],2),1,size(D,2));
                    data{m}(:,:,s) = D;
                end
            else
                data{m} = data{m}(:,:,rp);
            end
        end
        if LOSO % leave-one-subject-out analysis:
            subj_index = subj_index(rp);
            un = unique(subj_index);
            % Use subject-wise blocks as different CV folds:
            BL = [];
            for bl = 1:length(un)
                BL{bl} = find(subj_index == un(bl));
            end
            DataCV = divide_data_into_blocks(data,labels,nFolds,'threewayDA',saveMem,BL);
        else
            DataCV = divide_data_into_blocks(data,labels,nFolds,'threewayDA',saveMem);
        end
        outerLoopSize = size(DataCV.data,1);
        %    [UU,VV] = estimate_TF_projections(D,Lab,modelRank);
        %    features = extract_TF_features(D,UU,VV);
    case {'spectralLDA','temporalLDA','temporalRLDA',...
            'temporalLDAfull','spatialLDA','spatialRLDA',...
            'spatialLDAfull','temporalICA','temporalPCA',...
            'spatiotemporalLDA','spatiotemporalRLDA','spatiotemporalLDAfull'}
        rp = randperm(size(data,1));
        if strcmp(style,'spectralLDA')
            data = abs(data(rp,:,:));
        else
            data = data(rp,:,:);
        end
        labels = labels(rp);
        if LOSO % leave-one-subject-out analysis:
            subj_index = subj_index(rp);
            un = unique(subj_index);
            % Use subject-wise blocks as different CV folds:
            BL = [];
            for bl = 1:length(un)
                BL{bl} = find(subj_index == un(bl));
            end
            DataCV = divide_data_into_blocks(data,labels,nFolds,'bilinear',0,BL);
        else
            DataCV = divide_data_into_blocks(data,labels,nFolds,'bilinear');
        end
        outerLoopSize = size(DataCV.data,1);
    case {'spectralICA','temporalICA'}
        rp = randperm(size(data,1));
        if strcmp(style,'spectralICA')
            ma = max(abs(data(:)));
            data = abs(data(rp,:,:))/ma;
        else
            data = data(rp,:,:);
        end
        labels = labels(rp);
        if LOSO % leave-one-subject-out analysis:
            subj_index = subj_index(rp);
            un = unique(subj_index);
            % Use subject-wise blocks as different CV folds:
            BL = [];
            for bl = 1:length(un)
                BL{bl} = find(subj_index == un(bl));
            end
            DataCV = divide_data_into_blocks(data,labels,nFolds,'bilinear',0,BL);
        else
            DataCV = divide_data_into_blocks(data,labels,nFolds,'bilinear');
        end
        
        outerLoopSize = size(DataCV.data,1);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
