function [UU,VV] = estimate_TF_projections(D,Lab,modelRank,style)

for m = 1:length(D)
    Ra = size(D{m},2);
    if Ra < modelRank
%        warning(['Rank of the model ' num2str(modelRank) ' > dimension of the data, reducing rank to ' num2str(Ra)])
        modelRank = Ra;
    end
    Uf{m} = [];
    Vf{m} = [];
    nClasses = length(unique(Lab));
    if nClasses == 2
        nClasses = 1;
    end
    for n = 1:nClasses
        labelsTmp = zeros(size(Lab));
        labelsTmp(Lab == n) = 1;
        if strcmp(style,'sparse')
%            disp('computing sparse U,V...')
%            tic
            data_sSVD = permute(D{m},[3 1 2]);
            [U,V,S,spars_lev] = estimate_sparse_SVD(data_sSVD,labelsTmp+1,modelRank);
%            [U,V] = sparseSVDestimCV(meanDif,modelRank);
%            toc;
%            disp('done!')
            Uf{m} = [Uf{m} U];
            Vf{m} = [Vf{m} V];            
        elseif strcmp(style,'normal')
          
            D0 = nanmean(D{m}(:,:,labelsTmp == 0),3);
            D1 = nanmean(D{m}(:,:,labelsTmp == 1),3);

            meanDif = D1 - D0;            
            [U,DD,V] = svd(meanDif);
            Uf{m} = [Uf{m} U(:,1:modelRank)];
            Vf{m} = [Vf{m} V(:,1:modelRank)];
        elseif strcmp(style,'diagonal')
            [U,V,Uspher,Vspher] = regularized_threewayDA(D{m},labelsTmp,modelRank,style);
            Uf{m} = [Uf{m} U];
            Vf{m} = [Vf{m} V];            
        elseif strcmp(style,'full')
            [U,V,Uspher,Vspher] = regularized_threewayDA(D{m},labelsTmp,modelRank,style);
            Uf{m} = [Uf{m} U];
            Vf{m} = [Vf{m} V];            
        elseif strcmp(style,'fullorig')
            [U,V,Uspher,Vspher] = regularized_threewayDA(D{m},labelsTmp,modelRank,style);
            Uf{m} = [Uf{m} U];
            Vf{m} = [Vf{m} V];            
        elseif strcmp(style,'spatiotemporalLDA') | strcmp(style,'spatiotemporalRLDA') | strcmp(style,'spatiotemporalLDAfull')
            data_full = permute(D{m},[3 1 2]);
            nrChannels = size(data_full,2);
            nrTimepoints = size(data_full,1);
            switch style
                case 'spatiotemporalLDA'
                    regOpt = 'infinite';
                case 'spatiotemporalRLDA'
                    regOpt = 'diag'; 
                case 'spatiotemporalLDAfull'
                    regOpt = 'none';
            end
            Uf{m} = [];
            for c = 1:nrChannels
                data = squeeze(data_full(:,c,:))';
                [Uf{m}(:,c),J] = compute_LDA(data,labelsTmp,regOpt);
            end
            Vf{m} = [];
            for t = 1:nrTimepoints
                data = squeeze(data_full(t,:,:))';
                [Vf{m}(:,t),J] = compute_LDA(data,labelsTmp,regOpt);                
            end            
        else
            error
        end
    end
    if strcmp(style,'spatiotemporalLDA') | strcmp(style,'spatiotemporalRLDA') | strcmp(style,'spatiotemporalLDAfull')
        UfTmp = Uf;
        VfTmp = Vf;
    else
        UfTmp{m} = [];
        VfTmp{m} = [];
        for h = 1:modelRank
            UfTmp{m} = [UfTmp{m} Uf{m}(:,h:modelRank:size(Uf{m},2))];
            VfTmp{m} = [VfTmp{m} Vf{m}(:,h:modelRank:size(Uf{m},2))];
        end
    end
end
UU = UfTmp;
VV = VfTmp;
