function [U,V,S,spars_lev] = estimate_sparse_SVD(X,Lab,maxRank)

verbose = 0;

% 1SE criterion or max criterion:
max_crit = 1;
nrFolds = 10;
nr_lambda = 10;

[DataCV,BL] = divide_data_into_blocks(X,Lab,nrFolds,'bilinear');

results_full = [];
results_mean = [];
for m = 1:nrFolds
    if verbose
        disp(['Fold: ' num2str(m) '/' num2str(nrFolds)])
    end
    Xtrain = DataCV.data{m,1};
    Xtest = DataCV.data{m,2};
    labels_train = DataCV.labels{m,1};
    labels_test = DataCV.labels{m,2};
    
    M1 = squeeze(mean(Xtrain(labels_train==1,:,:),1));
    M2 = squeeze(mean(Xtrain(labels_train==2,:,:),1));
    A = M1 - M2;
    lambdas = linspace(1,sqrt(size(A,2)),nr_lambda);
%    lambdas = lambdas(2:end);
    [U,V,S,spars_lev] = sparseSVDestimCVFisher(A,maxRank,lambdas);
    % for each test sample, lambda value and factor, perform classification:
    for c = 1:length(lambdas)
        for k = 1:size(U{1},2) % each factor
            proj_mean1 = U{c}(:,k)'*M1*V{c}(:,k);
            proj_mean2 = U{c}(:,k)'*M2*V{c}(:,k);
            for n = 1:size(Xtest,1) % each data point
                Xt = squeeze(Xtest(n,:,:));
                proj_val = U{c}(:,k)'*Xt*V{c}(:,k);
                [mi,label_estim] = min(abs([proj_mean1 proj_mean2] - proj_val));
                correct = ( labels_test(n) == label_estim );
                results_full{k}(c,n,m) = correct;
            end
            results_mean{k} = squeeze(mean(results_full{k},2));
        end
    end
end

% find best lambdas for each factor:
acc2 = NaN*zeros(length(results_mean),size(results_mean{1},1));
accSE = acc2;
for k = 1:length(results_mean)
    acc2(k,:) = mean(results_mean{k},2);
    accSE(k,:) = std(results_mean{k},[],2)/sqrt(size(results_mean{k},2));
end
[ma,mai] = max(acc2,[],2);

clear ma mai
for m = 1:size(acc2,1) % for each factor
   [ma(m),mai(m)] = max(acc2(m,:));
   if max_crit % favor more accurate solution
        threshold = ma(m);
   else % favor sparser solution
       threshold = ma(m) - accSE(m,mai(m));
   end
   mai(m) = find(acc2(m,:) >= threshold,1,'last');
   ma(m) = acc2(m,mai(m));
end

%threshold = ma - accSE(mai);
% find first index within the range maxAccuracy - 1SE:
%min_idx = find(cvErr{i} <= threshold,1);
%min_err = cvErr{i}(min_idx);


Lam = lambdas(mai); % final lambdas for each factor

% Estimate final model:
M1 = squeeze(mean(X(Lab==1,:,:),1));
M2 = squeeze(mean(X(Lab==2,:,:),1));
A = M1 - M2;

way = 1;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Way 1:
switch way
    case 1 % This probably better way than way 2.
        % use optimal sparsity level for each factor:
        [U,V,S,spars_lev] = sparseSVDestimCVFisher(A,maxRank,Lam,'final');
        siz = [size(A) maxRank];
        V = V(:,1:min(siz));
        U = U(:,1:min(siz));
        S = diag(S)';
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Way 2:
    case 2
        % estimate factors for all the sparsity levels of the full data, then pick the
        % best factors.
        [U_final,V_final,S,spars_lev] = sparseSVDestimCVFisher(A,maxRank,lambdas);
        U = [];V = [];
        for k = 1:size(U_final{1},2) % for each factor
            % pick factor with optimal lambda:
            U = [U U_final{mai(k)}(:,k)];
            V = [V V_final{mai(k)}(:,k)];
        end
        siz = [size(A) maxRank];
        V = V(:,1:min(siz));
        U = U(:,1:min(siz));
end
