function Params = init_decoding_params(sourceFilenames,classNames,result_dir,triggerChannels)
% Set initial parameter struct for decoding.

if nargin < 3 || nargin > 4
    error('Number of inputs must be 3 or 4.')
end

% overwrite = 0: skip analysis if results already exist. 
% overwrite = 1: re-run analysis even if results already exist:
overwrite = 0; 

% randomize = 0: train true model
% randomize = 1: train null model by randomizing class labels
randomize = 0; 
% Index of the random model. Increase this to create more random models:
randomizeIndex = 1; 

% set parameters needed to create epochs:

winlen = 0.6000; % time-window length in seconds from the onset trigger of the stimulus
Fs = 200; % desired sampling frequency of the MEG signal in Hz (perhaps downsampled from original Fs)
skipfiltering = 0; % skipfiltering = 0: perform band-pass filtering, % skipfiltering = 1: skip band-pass filtering
lowcut =  0; % low-frequency cut-off frequency in Hz when using band-pass filtering
highcut = 45; % high-frequency cut-off frequency in Hz when using band-pass filtering
order = 3; % band-pass filter order
use_grad = 1; % use_grad = 0: use magnetometers, use_grad = 1: use gradiometers, use_grad = 2: use both
combineData = []; % combineData = []: don't combine epochs of the subjects, otherwise provide indices of the subjects to combine

% set decoding parameters:
classifiername = 'temporalLDA';
minfreq = 2;
maxfreq = 45;
CVfisher = 1;
skipscaling = 0;
modelRank = 10;
multiclass = 0;
winlen_channelwisetimeresolved_samples = 5;
winstep_channelwisetimeresolved_samples = 5;
subwinlen = 0.0020;
subwinlenSamp = 1;
classifierIndex = 1;
ica_on = 0; % ica_on = 0: no transforms, ica_on = 1: perform ICA, ica_on = -1: perform PCA
nrICs = 0;
samplingfactor = 1;
classifier = 'SLR';

generalParams.sourceFilenames = sourceFilenames;
generalParams.classNames = classNames;
generalParams.result_dir = result_dir;
generalParams.overwrite = overwrite;
generalParams.randomizeIndex = randomizeIndex;
generalParams.randomize = randomize;

generalParams.nrClasses = length(classNames);
generalParams.nrSubjects = length(sourceFilenames);



epochingParams.winlen = winlen;
epochingParams.Fs = Fs;
epochingParams.skipfiltering = skipfiltering;
epochingParams.lowcut = lowcut;
epochingParams.highcut = highcut;
epochingParams.order = order;
epochingParams.use_grad = use_grad;
epochingParams.combineData = combineData;
epochingParams.triggerChannels = triggerChannels;

decodingParams.classifiername = classifiername;
decodingParams.subwinlenSamp = subwinlenSamp;
decodingParams.minfreq = minfreq;
decodingParams.maxfreq = maxfreq;
decodingParams.CVfisher = CVfisher;
decodingParams.skipscaling = skipscaling;
decodingParams.modelRank = modelRank;
decodingParams.multiclass = multiclass;
decodingParams.classifierIndex = classifierIndex;
decodingParams.winlen_channelwisetimeresolved_samples = winlen_channelwisetimeresolved_samples;
decodingParams.winstep_channelwisetimeresolved_samples = winstep_channelwisetimeresolved_samples;
decodingParams.subwinlen = subwinlen;
decodingParams.ica_on = ica_on;
decodingParams.nrICs = nrICs;
decodingParams.samplingfactor = samplingfactor;
decodingParams.classifier = classifier;

Params.generalParams = generalParams;
Params.epochingParams = epochingParams;
Params.decodingParams = decodingParams;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check existence of source files and results-directory:
flag = 0;
for m = 1:length(sourceFilenames)
    if exist(sourceFilenames{m},'file') ~= 2
        warning(['Source file: "' sourceFilenames{m} '" does not exist.'])
        flag = 1;
    end
end
if flag
   error('Parameter validation failed.') 
else
   disp('All source files found.') 
end

flag = 0;
if exist(result_dir,'dir') == 0
    warning(['Results-directory not found, trying to create an empty folder...'])
    flag = 1;
    mkdir(result_dir)
end
if exist(result_dir,'dir') ~= 7
    error(['Could not create the result folder: "' result_dir '"'])
else
    if flag
        disp(['Results-directory created.'])
    else
        disp(['Results-directory found.'])        
    end
end

[pairs,nrClasses] = returnClasspairIndex(classNames);
Params.generalParams.pairs = pairs;
Params = update_directory_names(Params);


