function [DataCVTrain,dat,data_test,datO,UUfinal,VVfinal] = pick_training_and_validation_data(dat,data_test,lab,datOrig,nFoldsTrain,style,saveMem,modelRank,nrICs)

% pick training and validation data and extract features:

UUfinal = [];
VVfinal = [];
datO = [];

switch style
    case 'normal'
        DataCVTrain = divide_data_into_blocks(dat,lab,nFoldsTrain);
    case {'temporalthreewayDA','threewayDA','temporalthreewaySDA',...
            'temporalthreewayDAfullorig','temporalthreewayRDA','temporalthreewayDAfull'}
        DataCVTrain = divide_data_into_blocks(dat,lab,nFoldsTrain,'threewayDA',saveMem);
        switch style
            case 'temporalthreewaySDA'
                Styl = 'sparse';
            case 'temporalthreewayRDA'
                Styl = 'diagonal';
            case 'temporalthreewayDAfull'
                Styl = 'full';
            case 'temporalthreewayDAfullorig'
                Styl = 'fullorig';
            otherwise
                Styl = 'normal';
        end
        [UUfinal,VVfinal] = estimate_TF_projections(dat,lab,modelRank,Styl);
        datO = dat;
        dat = extract_TF_features(dat,UUfinal,VVfinal,Styl);
        data_test = extract_TF_features(data_test,UUfinal,VVfinal,Styl);
    case {'bilinear','temporalbilinear'}
        DataCVTrain = divide_data_into_blocks(dat,lab,nFoldsTrain,'threewayDA',saveMem);
        datO = dat;
    case {'spectralLDA','temporalLDA','spatiotemporalLDA','spatialLDA',...
            'temporalRLDA','spatiotemporalRLDA','spatialRLDA',...
            'temporalLDAfull','spatiotemporalLDAfull','spatialLDAfull'}
        DataCVTrain = divide_data_into_blocks(datOrig,lab,nFoldsTrain,'bilinear');
        % Extract spectral or temporal LDA features:
        % estimate coefficients from training data only:
        if strcmp(style,'spatiotemporalLDA') | ... 
                strcmp(style,'spatiotemporalRLDA') | ... 
                strcmp(style,'spatiotemporalLDAfull')            
            Styl = style;
            dat_tmp = dat;clear dat;dat{1} = dat_tmp;
            [UUfinal,VVfinal] = estimate_TF_projections(dat,lab,modelRank,Styl);
            dat = dat_tmp;
            dat = extract_TF_features(dat,UUfinal,VVfinal,Styl);
            data_test = extract_TF_features(data_test,UUfinal,VVfinal,Styl);
        else
            switch style
                case {'spatialRLDA','temporalRLDA'}
                    regOpt = 'diag';                    
                case {'spatialLDAfull','temporalLDAfull'}
                    regOpt = 'none';                    
                otherwise % for all other methods, use very heavy regularization (default)
                    regOpt = 'infinite';
            end
            switch style
                case {'spatialLDA','spatialRLDA','spatialLDAfull'}
                    projection_type = 'spatial_LDA';
                otherwise
                    projection_type = 'spectral_LDA';
            end
            [UUfinal,VVfinal] = estimate_projections(dat,lab,projection_type,modelRank,regOpt);
            % project training data:
            dat = project_data(dat,[],VVfinal,projection_type);
            % project test data:
            data_test = project_data(data_test,[],VVfinal,projection_type);
        end
    case {'spectralICA','temporalICA','temporalPCA'}
        DataCVTrain = divide_data_into_blocks(datOrig,lab,nFoldsTrain,'bilinear');
        % Extract spectral or temporal LDA features:
        % estimate coefficients from training data only:
        switch style
            case 'spectralICA'
                sty = 'spectral_ICA';
            case 'temporalICA';
                sty = 'temporal_ICA';
            case 'temporalPCA';
                sty = 'temporal_PCA';
        end
        [UUfinal,VVfinal,S] = estimate_projections(dat,lab,sty,nrICs);
        % project training data:
        dat = project_data(dat,[],VVfinal,sty);
        % project test data:
        data_test = project_data(data_test,[],VVfinal,sty);
end
